<?php
/**
 * Файл для описания класса хранилища логических БД, {@link mxl_core_logicbases}.
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания хранилища логических БД.
 * Используется для огранизации и работы с логическими БД на основе принципа
 * раздельной загрузки из директория.
 * Загрузка осуществляется из директория указаного в 
 * {@link mxl_cfg_path::$core_logicbases}, при попытке получить объект по имени.
 * 
 * Пример:
 * <code>
 * 	$MXL_LDBS = new mxl_core_logicbases();
 * 	$MXL_DB = $MXL_LDBS->getLogicBaseInst('logic_base_name');
 * </code>
 * 
 * Внимание!!!! Так как нет полноценной поддержки работы с БД, то объект 
 * подключения к бд регистрируется через вызов модуля php, в котором происходит
 * инициализация объекта. Регистраци осуществляется функцией 
 * {@link addLogicBase()}.
 * 
 * Пример создания экземпляра подключения к БД и его регистрация в хранилище:
 * <code>
 * define('IN_SYS', true);
 * define('USE_ENCRYPTED_PASSWORD', false);
 * 
 * $table_prefix = 'cp_';
 * $sys_table_prefix = '';
 * $dbhost = 'localhost';
 * $dbport = '';
 * $dbms = 'mysql';
 * $dbname = 'dbName';
 * $dbuser = 'root';
 * $dbpasswd = '';
 * 
 * require_once realpath(realpath(dirname(__FILE__)).'\..\..\..\libs\db\\' .$dbms. '.php');
 * $sql_db = 'dbal_' . $dbms;
 * 
 * $MXL_DB = new $sql_db();
 * $MXL_DB->sql_connect($dbhost, $dbuser, $dbpasswd, $dbname, $dbport, false, false);
 * 
 * unset($dbpasswd);
 * $this->addLogicBase('db_ceifx', 'Основная БД CEIFX', $MXL_DB);
 * </code>
  
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_core_logicbases {
	/**
	 * Массив для хранения объектов логических БД. Создается в конструкторе и 
	 * освобождается в деструкторе.
	 * @var array
	 */
	private $listLogicBases;
		
	/**
	 * Конструктор класса.
	 * @return mxl_core_logicbases
	 */
	public function __construct(){
		$this->listLogicBases = array();
	}
	
	/**
	 * Деструктор класса.
	 */
	function __destruct(){
		unset($this->listLogicBases);
	}

	/**
	 * Проверяет наличие логической БД с именем в хранилище, если есть то 
	 * выбрасывает исключение.
	 * @param $logicBaseName
	 * @throws Exception {@link mxl_cfg_exceptions::$core_logicbases_checkDontHave}
	 * @return boolean
	 */
	private function checkDontHave($logicBaseName){
		if (array_key_exists($logicBaseName, $this->listLogicBases)){
			$errMess = mxl_cfg_exceptions::$core_logicbases_checkDontHave;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $logicBaseName, $errMess);
				
			throw new Exception($errMess);
			return false;
		} else
			return true; 
	}
	
	/**
	 * Возвращает объект подключения к БД по имени.
	 * @param $logicBaseName
	 * @return sql_db
	 */
	public function getLogicBaseInst($logicBaseName){
		return $this->getLogicBase($logicBaseName)->getMXL_DB();
	}
	
	/**
	 * Регистрирует объект подключения к БД в хранилище логических БД.
	 * Перед регистрацией проверяет на наличие объекта с таким же именем,
	 * если объект есть то выбрасывает исключение. 
	 * @param $name
	 * @param $caption
	 * @param $mxl_lb
	 * @throws Exception {@link mxl_cfg_exceptions::$core_logicbases_checkDontHave}
	 */
	public function addLogicBase($name, $caption, $mxl_lb){
		if ($this->checkDontHave($name)){
			$LB = new mxl_core_logicbase($name, $caption, $mxl_lb);
			
			$this->listLogicBases[$name] = $LB;
		}
	}
	
	/* Функция формирует имя файла для загрузки из php, проверяет существование
	 * файла и вызывает загрузчик php в случае существования
	 * файла. В случае когда файл отсутствует выбрасывает исключение.
	 * 
	 * @param $logicBaseName
	 * @throws Exception {@link mxl_cfg_exceptions::$core_logicbases_loadLogicBase}
	 * @return boolean
	 */
	private function loadLogicBase($logicBaseName){
		$logicBaseFile = realpath(mxl_cfg_path::$core_logicbases
				.mxl_cfg_path::$separator.$logicBaseName.'.php');
		if (!file_exists($logicBaseFile)){
			$errMess = mxl_cfg_exceptions::$core_logicbases_loadLogicBase;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $logicBaseFile, $errMess);
					
			throw new Exception($errMess);	
			return false;
		}
		
		include $logicBaseFile;
		return true;	
	}
	
	/**
	 * Функция возвращает объект mxl_core_logicbase с именем $logicBaseName, 
	 * если таковой имеется в хранилище. Если объект не найдет то делается 
	 * попытка загрузить его из директория {@link mxl_cfg_path::$core_logicbases}.
	 * Если не удалось найти или загрузить объект с таким именем то выбрасывается
	 * исключение.  
	 * 
	 * @param $enumName - имя перечисления
	 * @throws Exception {@link mxl_cfg_exceptions::$core_logicbases_getLogicBase}
	 * @return mxl_core_logicbase
	 */
	public function getLogicBase($logicBaseName){
		if ((!array_key_exists($logicBaseName, $this->listLogicBases))&&
				(!$this->loadLogicBase($logicBaseName))){
					
			$errMess = mxl_cfg_exceptions::$core_logicbases_getLogicBase;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $logicBaseName, $errMess);
					
			throw new Exception($errMess);	
			return false;
		} 

		return $this->listLogicBases[$logicBaseName];
	} 
}
?>