<?php
/**
 * Файл для описания класса хранилища динамических параметров, {@link mxl_core_params}.
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания хранилища динамических параметров.
 * Используется для определения динамических параметров и последующей работы с
 * ними в тексте json-определения метаданных.
 * 
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_core_params{
	/**
	 * Массив для хранения параметров и значений.
	 * @var array()
	 */
	private $listParams;

	/**
	 * Конструктор хранилища параметров. 
	 * @param array $paramsArray - необязательный параметр, который передает
	 * 		в качестве массива значения для загрузки параметров 
	 * @return mxl_core_params
	 */
	public function __construct($paramsArray = ''){
		$this->listParams = array();
		
		if ($paramsArray != '')
			$this->addParamsFromArray($paramsArray);
	}
	
	/**
	 * Деструктор хранилища параметров. 
	 */
	public function __destruct(){
		unset($this->listParams);
	}
	
	/**
	 * Проверяет есть ли уже параметр с именем $paramName  
	 * @param string $paramName - имя параметра
	 * @return boolean
	 */
	public function isParamExists($paramName){
		return (boolean) array_key_exists($paramName, $this->listParams);
	}
	
	/**
	 * Функция добавляет новый параметр, предварительно производит проверку на
	 * наличие параметра с таким именем. Если найден параметр с именем $paramName
	 * то выбрасывается исключение с текстом:
	 * {@link mxl_cfg_exceptions::$core_params_addParam}
	 * @param string $paramName - имя параметра 
	 * @param string $paramValue - значение параметра
	 * @throws Exception {@link mxl_cfg_exceptions::$core_params_addParam}
	 */
	public function addParam($paramName, $paramValue){
		if ($this->isParamExists($paramName)){
			$errMess = mxl_cfg_exceptions::$core_params_addParam;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $paramName, $errMess);
				
			throw new Exception($errMess);
		} else
			$this->listParams[$paramName] = $paramValue;
	}
	
	/**
	 * Функция добавляет параметры из ассоциированного массива $paramsArray. В
	 * случае если нарушена структура массива или не указано значение для 
	 * элемента параметра выбрасывается исключение 
	 * {@link mxl_cfg_exceptions::$core_params_addParamsFromArray}.
	 * В случае если при добавлении параметра находится параметр с таким же 
	 * именем выбрасывается исключение 
	 * {@link mxl_cfg_exceptions::$core_params_addParam}.
	 * 
	 * @param array $paramsArray - ассоциированный массив из которого ведется 
	 * 		добавление параметров следующей структуры:
	 * <code>
	 * array('paramName1'=>'paramValue1'),
	 * 		 ...,
	 * 		 'paramNameN'=>'paramValueN'))
	 * </code>
	 * 
	 * @throws Exception {@link mxl_cfg_exceptions::$core_params_addParamsFromArray},
	 * 		{@link mxl_cfg_exceptions::$core_params_addParam}
	 */
	public function addParamsFromArray($paramsArray){
		foreach ($paramsArray as $paramName => $paramValue) {
			
			if (($paramName == '')){
				$errMess = mxl_cfg_exceptions::$core_params_addParamsFromArray;
				$x = 'paramName = "'. $paramName .'"';
						
				if (strpos($errMess, '#VTINS') !== false)
					$errMess = str_replace('#VTINS', $x, $errMess);
				
				throw new Exception($errMess);
			} else
				$this->addParam($paramName, $paramValue);
		}
	}
	
	/**
	 * Функция возвращает значение параметра, если не находит параметр то 
	 * выбрасывает исключение.
	 * @param string $paramName имя параметра
	 * @throws Exception {@link mxl_cfg_exceptions::$core_params_getParamValue}
	 * @return string
	 */
	public function getParamValue($paramName){
		if (! $this->isParamExists($paramName)){
			$errMess = mxl_cfg_exceptions::$core_params_getParamValue;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $paramName, $errMess);
				
			throw new Exception($errMess);
			return '';
		} else
			return $this->listParams[$paramName];
	}
	
	/**
	 * Функция проверяет: является ли набор параметров пустым 
	 * @return boolean
	 */
	public function isEmpty(){
		return (boolean) count($this->listParams) < 1;
	}
	
	/**
	 * Функция сохраняет набор параметров в формате json
	 * @return string
	 */
	public function saveToJSON(){
		/**
		 * необходимо два раза кодировать, что бы правильно отсылался запрос
		 */
		return json_encode(json_encode($this->listParams));
	}
	
	/**
	 * Функция загужает параметры из json
	 * @param string $json - строка для загрузки параметров в формате json
	 */
	public function loadFromJSON($json){
		/**
		 * один раз декодирует ExtJS, потому остается декодировать один раз
		 */
		$this->listParams = json_decode($json, true);
	}
	
	/**
	 * Функция возвращает параметры в виде ассоциированного массива:
	 * имя параметра => значение
	 * @return array
	 */
	public function getListParams(){
		return array_merge($this->listParams, array());	
	}
	
}
?>