<?php
/**
 * Файл для описания класса комбобокса с загрузкой значений из БД по SQL, 
 * {@link mxl_dict_comboboxdbsql}.
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2010, ООО НПО Метарекс
 */
/**
 * Класс для хранения информации по комбобоксу с загрузкой значений из БД по SQL.
 * Объект класса можно создать только путем загрузки из json-файла, которая 
 * осуществляется функцией {@link mxl_dict_combobox::loadComboboxFromJSON()}
 * при параметре "type" : "dbsql" в json файле.
 * Структура json-файла:<br>
 * <code>
 * {
 * "type"          : "dbsql",
 * "name"          : "имя комбобокса",
 * "logicBaseName" : "имя логической БД, из которой будут браться данные",
 * "query"         : "запрос для получения данных",
 * "where"         : "условие ограничения на отбор данных из запроса",
 * "order"         : "условие сортировки на данные их запроса",
 * "valueField"    : "поле из которого будут заполняться ключевые значения",
 * "displayField"  : "поле из которого будут отображаться данные",
 * "emptyText"     : "текст который будет показан когда не задано значение",
 * "inputTextMode" : false/true - возможность ввода текста для фильтрации значений,
 * "clearedMode"   : false/true - перевод сомбо в режим работы с кнопкой очистить
 * }
 * </code>
 * 
 * Пример json-файла:
 * <code>
 * {
 * "type"          : "dbsql",
 * "name"          : "countries",
 * "logicBaseName" : "db_ceifx",
 * "query"         : "select country_id, country_name from countries ",
 * "where"         : "",
 * "order"         : "",
 * "valueField"    : "country_id",
 * "displayField"  : "country_name",
 * "emptyText"     : "Select country...",
 * "inputTextMode" : false,
 * "clearedMode"   : false
 * }
 * </code>
 * 
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2010, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_dict_comboboxdbsql extends mxl_dict_combobox{
	/**
	 * Создает экземпляр класса на основе json файла комбобокса с типом 
	 * "type":"dbsql".
	 * @param $cfg - ассоциированный массив, построеный на основе json файла.
	 * @param $combo - экземпляр комбобокса, который передается из потомков
	 * класса для заполнения свойств текущего класса
	 * @return unknown_type
	 */
	public static function loadComboboxFromCfg($cfg, $combo = ''){
		if (!isset($combo)||($combo == '')) 
				$combo = new mxl_dict_comboboxdbsql($cfg['name'], $cfg['logicBaseName'], 
					$cfg['query'], $cfg['valueField'], $cfg['displayField'],
					$cfg['emptyText'], $cfg['where'], $cfg['order']);
		$combo = parent::loadComboboxFromCfg($cfg, $combo);
		return $combo;
	}
	
	/**
	 * Имя логической БД, из которой будут загружены значения комбобокса.
	 * @var string
	 */
	private $logicBaseName;
	
	/**
	 * запрос для получения значений значения комбобокса.
	 * @var string
	 */
	private $query;
	/**
	 * Условие для задания ограничения на отбор значений из Запроса на sql. 
	 * @var string
	 */
	private $where;
	
	/**
	 * Условие сортировки при отборе значений из запроса, на этапе отбора данных
	 * из БД.
	 * @var string
	 */
	private $order;
	
	/**
	 * Конструктор класса
	 * @param $name - имя комбобокса
	 * @param $logicBaseName - имя логческой БД из которой будет произведен
	 * 		отбор значений 
	 * @param $query - Запрос для получения значений из БД
	 * @param $valueField - имя ключевого поля из запроса
	 * @param $displayField - имя поля со значениями из запроса 
	 * @param $emptyText - текст отображаемый, когда не задано значение
	 * @param $where - условие для задания ограничения на отбор значений из 
	 * 		таблицы на sql
	 * @param $order - условие сортировки при отборе значений из БД, на этапе 
	 * 		отбора данных из БД.
	 * @return mxl_dict_comboboxdbsql
	 */
	public function __construct($name, $logicBaseName, $query, $valueField, 
			$displayField, $emptyText,	$where, $order = ''){
		parent::__construct($name, $valueField, $displayField, $emptyText);		
		
		$this->logicBaseName = $logicBaseName;
		$this->query         = $query;
		$this->where         = $where;
		$this->order         = $order;
	}
	
	/**
	 * Метод для возвращения имени логической БД, с которой работает комбобокс.
	 * @return string
	 */
	public function getLogicBaseName(){
		return $this->logicBaseName;
	}
	
	/**
	 * Метод для возвращения запроса для получения данных из БД.
	 * @return string
	 */
	public function getQuery(){
		return $this->query;
	}
	
	/**
	 * Метод для возвращения условия отбора значений.
	 * @return string
	 */
	public function getWhere(){
		return $this->where;
	}
	
	/**
	 * Метод для возвращения сортировки значений на этапе отбора данных из БД.
	 * @return string
	 */
	public function getOrder(){
		return $this->order;
	}
	
	/**
	 * Функция получения данных для заполнения комбобокса в формате json и их
	 * передачи на клиентскую часть.
	 * Функция формирует sql запрос на основании данных указанных в свойствах 
	 * класса, обращается к бд с целью получения данных и формирует текст ответа 
	 * в формате json.  
	 * Вывод осуществляется при помощи функции echo.
	 * 
	 * @TODO необходим обработчик ошибок получаемых от sql запросов 
	 */
	public function getJSON(){
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_DB = $MXL_LDBS->getLogicBaseInst($this->getLogicBaseName());
		
		header ('text/javascript');

		$query = $this->query;

		$WC = '';
		if ($this->getWhere())
			$WC .= $this->getWhere();
			
		if ($this->getQuerySeach()){
			if ($WC != '')
				$WC .= ' and ';
			$WC .= $this->getDisplayField().' like \'%'.$this->getQuerySeach().'%\'';
		}
		
		if ($WC != '')
			$query .= ' where '.$WC;
			
		$query .= ($this->getOrder() ? ' order by '.$this->getOrder() : '');

		$result = $MXL_DB->sql_query($query);
		$cnt = 0;
		while ($r = $MXL_DB->sql_fetchrow($result)){
			$cnt++;
            //вызов рендерера для строки
            if ($this->getRenderer()){
                $r = mxl_dict_renderer::execute_renderer ($this->getRenderer(), 
                       mxl_core_metadatas::instance()->getCurrParams(), $r);
            }

			$rows [] = $r;
		}
		$MXL_DB->sql_freeresult($result);

		if ($cnt > 0){
			echo '({"total":"'.$cnt.'","rows":'.json_encode($rows).'})';
		}else{
			echo '({"total":"0", "rows":[]})';
		}
		return true;
	}
	
	/**
	 * Функция которая проверяет тип значения и если он строковый то
	 * дополняет его апострофами с целью вывода в текст sql выражения.
	 * @param $value - проверяемое значение
	 * @return unknown_type
	 */
	private function getValueToWC($value){
		if (is_string($value))
			return '\''.$value.'\'';
		else 
			return $value;
	}
	
	/**
	 * Функция формирует запрос к БД, с целью получения значения для ключа. 
	 * @param $startValue - ключ для которого необходимо найти значение.
	 * @return string
	 */
	public function getStartDisplay($startValue){
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_DB = $MXL_LDBS->getLogicBaseInst($this->getLogicBaseName());
		
		$query = $this->query.' where '. $this->getValueField().' = '
				.$this->getValueToWC($startValue);

		$displVal = '';
		$result = $MXL_DB->sql_query($query);
		if ($r = $MXL_DB->sql_fetchrow($result))
			$displVal = $r[$this->getDisplayField()];
			
		$MXL_DB->sql_freeresult($result);
		
		return $displVal;
	}
}

?>