<?php
/**
 * Файл для описания класса "Списка" с загрузкой значений из Запроса к БД, 
 * {@link mxl_dict_listdbsql}.
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2010, ООО НПО Метарекс
 */
/**
 * Класс для хранения информации о "Списке" с загрузкой значений из БД.
 * Объект класса можно создать только путем загрузки из json-файла, которая 
 * осуществляется функцией {@link mxl_dict_list::loadFromJSON()}
 * при параметре "type" : "dbsql" в json файле.
 * Структура json-файла:<br>
 * <code>
 * {
 * "type"          : "dbsql",
 * "name"          : "имя списка",
 *
 * "logicBaseName" : "имя логической БД, из которой будут браться данные",
 * "query"         : "запрос для получения значений из БД",
 * "valueField"    : "поле из которого будут заполняться ключевые значения",
 * "displayField"  : "поле из которого будут отображаться данные",
 * "where"         : "условие ограничения на отбор данных",
 * "order"         : "условие сортировки на данные",
 * }
 * </code>
 * 
 * Пример json-файла:
 * <code>
 * {
 * "type"          : "dbsql",
 * "name"          : "countries",
 * "logicBaseName" : "db_ceifx",
 * "query"         : "select country_id, country_name from countries",
 * "valueField"    : "country_id",
 * "displayField"  : "country_name",
 * "where"         : "",
 * "order"         : "",
 * }
 * </code>
 * 
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2010, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_dict_listdbsql extends mxl_dict_list{
	public static function loadListFromCfg($cfg, $list = ''){
		if (!isset($list)||($list == '')) 
			$list = new mxl_dict_listdbsql($cfg['name'], $cfg['logicBaseName'], 
				$cfg['query'], $cfg['valueField'], $cfg['displayField'],
				$cfg['where'], $cfg['order']);
		$list = parent::loadListFromCfg($cfg, $list);
		return $list;
	}
	
	/**
	 * Имя логической БД, из которой будут загружены значения комбобокса.
	 * @var string
	 */
	private $logicBaseName;
	
	/**
	 * Запрос для получения значений списка.
	 * @var string
	 */
	private $query;
	/**
	 * Условие для задания ограничения на отбор значений из таблицы на sql. 
	 * @var string
	 */
	private $where;
	
	/**
	 * Условие сортировки при отборе значений из БД, на этапе отбора данных
	 * из БД.
	 * @var string
	 */
	private $order;
	
	/**
	 * Конструктор класса
	 * @param string $name - имя "Списка"
	 * @param string $logicBaseName - имя логческой БД из которой будет произведен
	 * 		отбор значений 
	 * @param string $query - Запрос на получение значений
	 * @param string $valueField - имя ключевого поля из запроса
	 * @param string $displayField - имя поля со значениями из запроса 
	 * @param string $where - условие для задания ограничения на отбор значений из 
	 * 		запроса на sql
	 * @param string $order - условие сортировки при отборе значений из БД, на этапе 
	 * 		отбора данных из БД.
	 * @return mxl_dict_listdb
	 */
	public function __construct($name, $logicBaseName, $query, $valueField, $displayField, 
			$where, $order = ''){
		parent::__construct($name, $valueField, $displayField);		
		
		$this->logicBaseName = $logicBaseName;
		$this->query         = $query;
		$this->where         = $where;
		$this->order         = $order;
	}
	
	/**
	 * Метод для возвращения имени логической БД, с которой работает комбобокс.
	 * @return string
	 */
	public function getLogicBaseName(){
		return $this->logicBaseName;
	}
	
	/**
	 * Метод для возвращения Запроса на получения данных из БД.
	 * @return string
	 */
	public function getQuery(){
		return $this->query;
	}
	
	/**
	 * Метод для возвращения условия отбора значений.
	 * @return string
	 */
	public function getWhere(){
		return $this->where;
	}
	
	/**
	 * Метод для возвращения сортировки значений на этапе отбора данных из БД.
	 * @return string
	 */
	public function getOrder(){
		return $this->order;
	}
	
	/**
	 * Функция получения данных для заполнения комбобокса в формате json и их
	 * передачи на клиентскую часть.
	 * Функция формирует sql запрос на основании данных указанных в свойствах 
	 * класса, обращается к бд с целью получения данных и формирует текст ответа 
	 * в формате json.  
	 * Вывод осуществляется при помощи функции echo.
	 */
	public function getJSON(){
		$MXL_LDBS = new mxl_core_logicbases();
		$MXL_DB = $MXL_LDBS->getLogicBaseInst($this->getLogicBaseName());
		
		header ('text/javascript');

		$query = $this->query;

		$query .= ($this->getWhere() ? ' where '   .$this->getWhere() : '');
		$query .= ($this->getOrder() ? ' order by '.$this->getOrder() : '');
			
		$result = $MXL_DB->sql_query($query);
		$cnt = 0;
		while ($r = $MXL_DB->sql_fetchrow($result)){
			$cnt++;
            //вызов рендерера для строки
            if ($this->getRenderer()){
                $r = mxl_dict_renderer::execute_renderer ($this->getRenderer(), 
                       mxl_core_metadatas::instance()->getCurrParams(), $r);
            }

			$rows [] = $r;
		}
		$MXL_DB->sql_freeresult($result);

		if ($cnt > 0){
			echo '({"total":"'.$cnt.'","rows":'.json_encode($rows).'})';
		}else{
			echo '({"total":"0", "rows":[]})';
		}
		return true;
	}
}
?>