<?php
/**
 * Файл для описания класса валидатора, {@link mxl_dict_validator}.
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания валидатора.
 * Объект "валидатор" можно создать только путем загрузки из json-файла,
 * которая осуществляется функцией {@link mxl_dict_validator::loadFromJSON()}
 * Структура json-файла определения перечисления:<br>
 * <code>
 * "before_validate" : "Имя сниппета валидатора, выполняемого перед валидацией",
 * "after_validate"  : "Имя сниппета валидатора, выполняемого после валидациии",
 * 
 * "rules" : {
 * секция общих правил - которые не требуют знания о природе поля из описания метаданного
 * 	"common" : {
 *      правило required - проверяет набор полей на обязательность наличия значения.
 *      в качестве параметра передается список полей разделенных ';'
 * 		"required"      : "имя поля 1; имя поля 2;"
 * 	},
 * секция правил для грида - требуют знания по какому гриду строится правило валидации
 * 	"dbgrid" : {
 * 		"check_type" : "amount;ngroup;",
 * 		"unique"     : "fee_id;"
 * 	}
 * }
 * }
 * </code>
 * 
 * Пример json-файла:
 * <code> 
 * {
 * "before_validate" : "fees1_before",
 * "after_validate"  : "",
 * 
 * "rules" : {
 * 	"common" : {
 * 		"required"      : "product_type;product_id:Продукты;amount;fee_type;"
 * 	},
 * 	"dbgrid" : {
 * 		"required"      : "product_type;product_id;amount;fee_type;"
 * 	}
 * }
 * }
 * </code>
 * 
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_dict_validator extends mxl_core_metadata{
	
	/**
	 * Статическая функция читает json-файл и создает объект валидатора.
	 * @param string $metaDataName - имя валидатора
	 * @param mxl_core_params $params - динамические параметры
	 * @return mxl_dict_validator
	 */
	public static function loadFromJSON($metaDataName, mxl_core_params $params){
		$fileName = mxl_core_metadata::getJSONFileName(
				mxl_cfg_path::$dict_validators,	$metaDataName);
		
		$cfg = mxl_utils_json::readArrayFromFile($fileName, $params);

		
		$bef_A = (isset($cfg['beforeAllValidate']) ? $cfg['beforeAllValidate'] : '');
		$bef_v = (isset($cfg['beforeValidate'])    ? $cfg['beforeValidate']    : '');
		$bef_d = (isset($cfg['beforeDelete'])      ? $cfg['beforeDelete']      : '');
		$aft_v = (isset($cfg['afterValidate'])     ? $cfg['afterValidate']     : '');
		$rules = $cfg['rules'];
		
		
		$validator = new mxl_dict_validator($metaDataName, $bef_A, $bef_v, $bef_d, 
				$aft_v);
		foreach ($rules as $section=>$values){
			foreach ($values as $ruleName => $value)
				$validator->addValidateRule($section, $ruleName, $value);
		}
		return $validator;
	}

	/**
	 * Список правил
	 * 
	 * @var array
	 */
	private $listRules;
	
	/**
	 * Тело сниппета, выполняющегося до валидации всего массива измененных 
	 * данных
	 * 
	 * @var string
	 */
	private $beforeAllValidate;
	/**
	 * Тело сниппета, выполняющегося до валидации построчно
	 * 
	 * @var string
	 */
	private $beforeValidate;
	
	/**
	 * Тело сниппета, выполняющегося после валидации построчно
	 * 
	 * @var string
	 */
	private $afterValidate;
	
	/**
	 * Тело сниппета, выполняющегося перед удалением. 
	 * 
	 * @var string
	 */
	private $beforeDelete;
	
	
	/**
	 * Конструктор класса. Создает объект класса и инициализирует начальными 
	 * значениями.
	 * 
	 * @param string $name - имя валидатора
	 * @param string beforeAllValidate - тело сниппета, выполняющегося до 
	 *	 	валидации по всему набору данных
	 * @param string $beforeValidate - тело сниппета, выполняющегося до 
	 * 		валидации построчно
	 * @param string $beforeDelete - тело сниппета, выполняющегося перед удалением
	 * @param string $afterValidate - тело сниппета, выполняющегося после валидации
	 * @return mxl_dict_validator
	 */
	function __construct($name, $beforeAllValidate = '', $beforeValidate = '', 
			$beforeDelete = '', $afterValidate = ''){
		parent::__construct($name);		
		
		$this->beforeAllValidate = $beforeAllValidate;
		$this->beforeValidate    = $beforeValidate;
		$this->afterValidate     = $afterValidate;
		$this->beforeDelete      = $beforeDelete;

		$this->listRules = array();
		foreach (mxl_dict_validator_checker::$possibleRules as $key=>$value)
			$this->listRules[$key] = array();
	}
	
	/**
	 * Деструктор валидатора.
	 */
	public function __destruct(){
		foreach (mxl_dict_validator_checker::$possibleRules as $key=>$value)
			unset($this->listRules[$key]);
		unset($this->listRules);
	}
	
	/**
	 * Метод проверяет поддерживается ли правило системой. 
	 * 
	 * @param string $sectionName - имя секции json-словаря валидатора
	 * @param string $ruleName - имя правила
	 * @throws Exception {@link mxl_cfg_exceptions::$dict_validator_checkRule}
	 * @return boolean
	 */
	public function checkRule($sectionName, $ruleName){
		foreach (mxl_dict_validator_checker::$possibleRules as $section=>$rules)
			if ($section == $sectionName)
				foreach ($rules as $rule)
					if ($rule == $ruleName)
						return true;

		$errMess = mxl_cfg_exceptions::$dict_validator_checkRule;
		if (strpos($errMess, '#VTINS') !== false)
			$errMess = str_replace('#VTINS', $sectionName, $errMess);
		if (strpos($errMess, '#VTINS1') !== false)
			$errMess = str_replace('#VTINS1', $ruleName, $errMess);
					
		throw new Exception($errMess);
		return false;
	}
	
	/**
	 * Метод проверяет существование правила с таким именем в списке правил валидатора.
	 * Если правила с таким именем нет, возвращает true, иначе выбрасывает исключение 
	 * и возвращает false.
	 * 
	 * @param string $sectionName - имя секции json-словаря валидатора
	 * @param string $ruleName - имя правила
	 * @throws Exception {@link mxl_cfg_exceptions::$core_metadatas_checkDontHave}
	 * @return boolean
	 */
	private function checkDontHave($sectionName, $ruleName){
		if (!$this->checkRule($sectionName, $ruleName))
			return false;
		if (array_key_exists($ruleName, $this->listRules[$sectionName])){
			
			$errMess = mxl_cfg_exceptions::$core_metadatas_checkDontHave;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS',  $ruleName, $errMess);
			if (strpos($errMess, '#VTINS1') !== false)
				$errMess = str_replace('#VTINS1', $sectionName, $errMess);
				
			throw new Exception($errMess);

			return false;
		} else
			return true; 
	}
	
	/**
	 * Метод добавляет правило в список правил валидатора.
	 * 
	 * @param string $sectionName - имя секции json-словаря валидатора 
	 * @param string $ruleName - имя правила
	 * @param string $value - правило
	 */
	private function addValidateRule($sectionName, $ruleName, $value){
		if ($this->checkDontHave($sectionName, $ruleName))
			$this->listRules[$sectionName][$ruleName] = $value;
	}
	
	/**
	 * Метод для получения имени валидатора выполняющегося перед всей 
	 * валидацией по всему набору модифицированных записей
	 * @return string
	 */
	public function getBeforeAllValidate(){
		return $this->beforeAllValidate;
	}
	
	/**
	 * Метод для получения имени валидатора выполняющегося перед  
	 * валидацией по строчно
	 * @return string
	 */
	public function getBeforeValidate(){
		return $this->beforeValidate;
	}

	/**
	 * Метод для получения имени валидатора выполняющегося перед 
	 * удалением записи
	 * @return string
	 */
	public function getBeforeDelete(){
		return $this->beforeDelete;
	}

	/**
	 * Метод для получения имени валидатора выполняющегося после
	 * валидации по строчно
	 * @return string
	 */
	public function getAfterValidate(){
		return $this->afterValidate;
	}
	
	public function getRules(){
		return $this->listRules;	
	}
}
?>