<?php
/**
 * Файл для описания базового класса визуальных компонентов (виджетов), 
 * {@link mxl_widget_widget}.
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
if (!defined('NL')){
	#define('NL', 	"<br>");
	define('NL', 	"\n");
}

/**
 * Абстрактный базовый класс описания визуальных компонентов (виджетов).
 * Используется для определения базовых свойст и методов, наличие которых 
 * обязательно для всех виджетов.
 * 
 * @package mxl_widget
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
abstract class mxl_widget_widget{
	/**
	 * Имя виджета, будет использовано при построении визуальной части и будет 
	 * соответствовать имени компонента.
	 * @var string
	 */
	private $name;
	
	/**
	 * Переменная для сохранения параметров, которые передаются через виджеты,
	 * что бы при получения JS кода их трансформировать в BaseParams 
	 * @var mxl_core_params
	 */
	private $params;
	
	/**
	 * Смещение времени от серверного в часах. 
	 * @var float
	 */
	private $timeShift;
	
	/**
	 * Массив для хранения описаний переопределенных обработчиков событий
	 * @var array()
	 */
	private $listEventHandler;
	
	/**
	 * Метод установки переменной для сохранения параметров, которые передаются
	 * через виджеты, что бы при получении JS кода трансформировать их в 
	 * BaseParams
	 * @param mxl_core_params $params
	 */
	public function setParams(mxl_core_params $params){
		$this->params = $params;
	} 
	
	/**
	 * Метод получения переменной для сохранения параметров, которые передаются
	 * через виджеты, что бы при получении JS кода трансформировать их в 
	 * BaseParams
	 * @return mxl_core_params $params
	 */
	public function getParams(){
		return $this->params;
	} 
	
	/**
	 * Абстракная функция для получения ajax кода инициализации и работы компонента.
	 * Генерируемый код делится на несколько частей, которые будут размещены в 
	 * определенном порядке после генерации всего ajax кода.
	 * 
	 * @param string $declarePartCommon - декларативная общая часть, размещается 1-ой.
	 * @param string $declarePartPers - декларативная часть для компонента, размещается 2-ой
	 * @param string $runPartFirst - часть которая будет выполнена 1-ой, размещается 3-ей, 
	 * 		но 1-ой в части выполнения startAutoGenScript
	 * @param string $runPartLast - часть которая будет выполнена последней, размещается 4-ей, 
	 * 		но 2-ой в части выполнения startAutoGenScript
	 * @param string $funcsAfterInit - функции которые будут выполнены поле инициализации,
	 * 		размещается 4-ой.
	 * @param boolean $append_SID - параметр (костыль) определяющий передавать ли $_SID через
	 * 		baseParam и необходимо ли определять функцию для получения _SID
	 */
	abstract function getJSCode(&$declarePartCommon, &$declarePartPers, 
			&$runPartFirst, &$runPartLast, &$funcsAfterInit, $append_SID = false);
	
	/**
	 * Конструктор класса.
	 * @param string $name
	 * @return mxl_widget_widget
	 */
	public function __construct($name){
		$this->name      = $name;
		$this->timeShift = 0;
		
		$this->listEventHandler = array();
	}
	
	/**
	 * Деструктор класса.
	 */
	function __destruct(){
		unset($this->listEventHandler);
	}
	
	/**
	 * Функция получения имени виджета.
	 * @return string
	 */
	public function getName(){
		return $this->name;
	}
	
	/**
	 * Смещение времени относительно серверного в часах. 
	 * @param float $timeShift
	 */
	public function setTimeShift($timeShift){
		$this->timeShift = $timeShift;
	}
	
	/**
	 * Смещение времени относительно серверного в часах. 
	 * @return float
	 */
	public function getTimeShift(){
		return $this->timeShift;
	}
	
	/**
	 * Функция проверки типа обработчика события визуального компонента.
	 * В случае удачной проверки возвращает true;
	 * @param string $type - проверяемый тип
	 * @return boolean
	 */
	abstract function checkEventHandlerType($type);
	
	/**
	 * Метод переопределения Стандартного обработчика событий.  
	 * @param string $type - тип обработчика события. Для каждого компонента 
	 * 		предусмотрен свой набор типов обработчиков событий.
	 * @param string $functionName - имя JS функции, которая переопределяет 
	 * 		стандартный обработчик события.
	 * @throws Exception {@link mxl_cfg_exceptions::$widget_widget_setEventHandler}
	 */ 
	public function setEventHandler($type, $functionName){
		if (!$this->checkEventHandlerType($type)){
			$errMess = mxl_cfg_exceptions::$widget_widget_setEventHandler;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $type, $errMess);
				
			throw new Exception($errMess);
			return false;
		}
		$eh = new mxl_widget_eventhandler($type, $functionName);
		$this->listEventHandler[$type] = $eh;
	}
	
	/**
	 * Метод проверки наличия переопределенного обработчика с типом $type
	 * @param string $type - проверяемый тип обработчика события
	 * @return boolean
	 */
	public function hasEventHandler($type){
		if (!$this->checkEventHandlerType($type)){
			$errMess = mxl_cfg_exceptions::$widget_widget_setEventHandler;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $type, $errMess);
				
			throw new Exception($errMess);
			return false;
		};
		
		return isset($this->listEventHandler[$type]);
	}

	/**
	 * Метод получения имени функции переобределенного обработчика с типом $type
	 * @param string $type - тип обработчика события для которого ищется функция
	 * @return string
	 */
	public function getEventHandlerFunctName($type){
		if (!$this->checkEventHandlerType($type)){
			$errMess = mxl_cfg_exceptions::$widget_widget_setEventHandler;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $type, $errMess);
				
			throw new Exception($errMess);
			return false;
		};
		
		$eh = $this->listEventHandler[$type];
		
		return $eh->getFunctionName();
	}

	public function prepareEventHandler($type, $defaultName){
		return ($this->hasEventHandler($type) ? 
				$this->getEventHandlerFunctName($type):	
				$defaultName);
	}
}
?>